# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Circle"

def parse_circle(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[2] in ("deposit", "internal_switch_currency", "switch_currency"):
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[7].strip('£').split(' ')[0],
                                                 buy_asset=in_row[8],
                                                 sell_quantity=in_row[5].strip('£').split(' ')[0],
                                                 sell_asset=in_row[6],
                                                 wallet=WALLET)
    elif in_row[2] == "spend":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[5].strip('£').split(' ')[0],
                                                 sell_asset=in_row[6],
                                                 sell_value=in_row[7].strip('£') if in_row[8] == \
                                                                                     config.CCY \
                                                                                 else None,
                                                 wallet=WALLET)
    elif in_row[2] == "receive":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[7].strip('£').split(' ')[0],
                                                 buy_asset=in_row[8],
                                                 buy_value=in_row[5].strip('£') if in_row[6] == \
                                                                                    config.CCY \
                                                                                else None,
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

DataParser(DataParser.TYPE_EXCHANGE,
           "Circle",
           ['Date', ' Reference ID', ' Transaction Type', ' From Account', ' To Account',
            ' From Amount', ' From Currency', ' To Amount', ' To Currency', ' Status'],
           worksheet_name="Circle",
           row_handler=parse_circle)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal, ROUND_UP

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Poloniex"

PRECISION = Decimal('0.00000000')

def parse_poloniex_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[3] == "Buy":
        fee_quantity = Decimal(in_row[5]) - Decimal(in_row[5]) * \
                (Decimal(in_row[7].replace('%', '')) / Decimal(100))
        fee_quantity = fee_quantity.quantize(PRECISION, rounding=ROUND_UP)

        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[1].split('/')[0],
                                                 sell_quantity=in_row[6],
                                                 sell_asset=in_row[1].split('/')[1],
                                                 fee_quantity=Decimal(in_row[5]) - fee_quantity,
                                                 fee_asset=in_row[1].split('/')[0],
                                                 wallet=WALLET)
    elif in_row[3] == "Sell":
        fee_quantity = Decimal(in_row[6]) - Decimal(in_row[6]) * \
                (Decimal(in_row[7].replace('%', '')) / Decimal(100))
        fee_quantity = fee_quantity.quantize(PRECISION, rounding=ROUND_UP)

        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[6],
                                                 buy_asset=in_row[1].split('/')[1],
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[1].split('/')[0],
                                                 fee_quantity=Decimal(in_row[6]) - fee_quantity,
                                                 fee_asset=in_row[1].split('/')[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(3, parser.in_header[3], in_row[3])

def parse_poloniex_deposits_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if "COMPLETE:" in in_row[4]:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[2],
                                                 sell_asset=in_row[1],
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[2],
                                                 buy_asset=in_row[1],
                                                 wallet=WALLET)

def parse_poloniex_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                             data_row.timestamp,
                                             sell_quantity=Decimal(in_row[2]) - Decimal(in_row[3]),
                                             sell_asset=in_row[1],
                                             fee_quantity=in_row[3],
                                             fee_asset=in_row[1],
                                             wallet=WALLET)

DataParser(DataParser.TYPE_EXCHANGE,
           "Poloniex Trades",
           ['Date', 'Market', 'Category', 'Type', 'Price', 'Amount', 'Total', 'Fee', 'Order Number',
            'Base Total Less Fee', 'Quote Total Less Fee'],
           worksheet_name="Poloniex T",
           row_handler=parse_poloniex_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "Poloniex Deposits/Withdrawals",
           ['Date', 'Currency', 'Amount', 'Address', 'Status'],
           worksheet_name="Poloniex D,W",
           row_handler=parse_poloniex_deposits_withdrawals)

DataParser(DataParser.TYPE_EXCHANGE,
           "Poloniex Withdrawals",
           ['Date', 'Currency', 'Amount', 'Fee Deducted', 'Amount - Fee', 'Address', 'Status'],
           worksheet_name="Poloniex W",
           row_handler=parse_poloniex_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Coinfloor"

def parse_coinfloor_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[7] == "Buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3],
                                                 buy_asset=in_row[1],
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[2],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[2],
                                                 wallet=WALLET)
    elif in_row[7] == "Sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[2],
                                                 sell_quantity=in_row[3],
                                                 sell_asset=in_row[1],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[2],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(7, parser.in_header[7], in_row[7])

def parse_coinfloor_deposits_withdrawals(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[3] == "Deposit":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[1],
                                                 buy_asset=in_row[2],
                                                 wallet=WALLET)
    elif in_row[3] == "Withdrawal":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[1],
                                                 sell_asset=in_row[2],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(3, parser.in_header[3], in_row[3])

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinfloor Trades",
           ['Date & Time', 'Base Asset', 'Counter Asset', 'Amount', 'Price', 'Total', 'Fee',
            'Order Type'],
           worksheet_name="Coinfloor T",
           row_handler=parse_coinfloor_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinfloor Deposits/Withdrawals",
           ['Date & Time', 'Amount', 'Asset', 'Type'],
           worksheet_name="Coinfloor D,W",
           row_handler=parse_coinfloor_deposits_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

import logging
from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import DataParserError, UnexpectedTypeError, \
                         MissingValueError, MissingComponentError

WALLET = "Gatehub"

log = logging.getLogger()

def parse_gatehub(data_rows, parser):
    for data_row in data_rows:
        if data_row.parsed:
            continue

        try:
            parse_gatehub_row(data_rows, parser, data_row)
        except DataParserError as e:
            data_row.failure = e

def parse_gatehub_row(data_rows, parser, data_row):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])
    data_row.parsed = True

    t_type = ""
    buy_quantity = None
    buy_asset = ""
    sell_quantity = None
    sell_asset = ""
    fee_quantity = None
    fee_asset = ""

    if not in_row[3]:
        raise MissingValueError(3, parser.in_header[3])

    if in_row[2] == "payment":
        if Decimal(in_row[3]) < 0:
            t_type = TransactionOutRecord.TYPE_WITHDRAWAL
            sell_quantity = abs(Decimal(in_row[3]))
            sell_asset = in_row[4]
        else:
            t_type = TransactionOutRecord.TYPE_DEPOSIT
            buy_quantity = in_row[3]
            buy_asset = in_row[4]

        fee_quantity, fee_asset = find_same_tx(data_rows, in_row[1],
                                               "ripple_network_fee")
    elif in_row[2] == "exchange":
        t_type = TransactionOutRecord.TYPE_TRADE
        if Decimal(in_row[3]) < 0:
            sell_quantity = abs(Decimal(in_row[3]))
            sell_asset = in_row[4]

            buy_quantity, buy_asset = find_same_tx(data_rows, in_row[1],
                                                   "exchange")
        else:
            buy_quantity = in_row[3]
            buy_asset = in_row[4]

            sell_quantity, sell_asset = find_same_tx(data_rows, in_row[1],
                                                     "exchange")

        if sell_quantity is None or buy_quantity is None:
            raise MissingComponentError(1, parser.in_header[1], in_row[1])

        fee_quantity, fee_asset = find_same_tx(data_rows, in_row[1],
                                               "ripple_network_fee")
    elif in_row[2] == "ripple_network_fee":
        # Fees which are not associated with a payment or exchange are added
        # as a Spend
        t_type = TransactionOutRecord.TYPE_SPEND
        sell_quantity = abs(Decimal(in_row[3]))
        sell_asset = in_row[4]
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

    data_row.t_record = TransactionOutRecord(t_type,
                                             data_row.timestamp,
                                             buy_quantity=buy_quantity,
                                             buy_asset=buy_asset,
                                             sell_quantity=sell_quantity,
                                             sell_asset=sell_asset,
                                             fee_quantity=fee_quantity,
                                             fee_asset=fee_asset,
                                             wallet=WALLET)

def find_same_tx(data_rows, tx_hash, tx_type):
    quantity = None
    asset = ""

    data_rows = [data_row for data_row in data_rows
                 if data_row.in_row[1] == tx_hash and not data_row.parsed]
    for data_row in data_rows:
        if tx_type == data_row.in_row[2]:
            quantity = abs(Decimal(data_row.in_row[3]))
            asset = data_row.in_row[4]
            data_row.timestamp = DataParser.parse_timestamp(data_row.in_row[0])
            data_row.parsed = True
            break

    return quantity, asset

DataParser(DataParser.TYPE_EXCHANGE,
           "GateHub (Ripple)",
           ['Time', 'TX hash', 'Type', 'Amount', 'Currency', 'Currency Issuer Address',
            'Currency Issuer Name', 'Balance'],
           worksheet_name="Gatehub",
           all_handler=parse_gatehub)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Cryptopia"

def parse_cryptopia_deposits(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[7])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                             data_row.timestamp,
                                             buy_quantity=in_row[2],
                                             buy_asset=in_row[1],
                                             wallet=WALLET)

def parse_cryptopia_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[7])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                             data_row.timestamp,
                                             sell_quantity=Decimal(in_row[2]) - Decimal(in_row[3]),
                                             sell_asset=in_row[1],
                                             fee_quantity=in_row[3],
                                             fee_asset=in_row[1],
                                             wallet=WALLET)

def parse_cryptopia_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[7])

    if in_row[2] == "Buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[4],
                                                 buy_asset=in_row[1].split('/')[0],
                                                 sell_quantity=Decimal(in_row[3]) * \
                                                               Decimal(in_row[4]),
                                                 sell_asset=in_row[1].split('/')[1],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[1].split('/')[0],
                                                 wallet=WALLET)
    elif in_row[2] == "Sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[3]) * \
                                                              Decimal(in_row[4]),
                                                 buy_asset=in_row[1].split('/')[1],
                                                 sell_quantity=in_row[4],
                                                 sell_asset=in_row[1].split('/')[0],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[1].split('/')[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

DataParser(DataParser.TYPE_EXCHANGE,
           "Cryptopia Deposits",
           ['#', 'Currency', 'Amount', 'Status', 'Type', 'Transaction', 'Conf.', 'Timestamp'],
           worksheet_name="Cryptopia D",
           row_handler=parse_cryptopia_deposits)

DataParser(DataParser.TYPE_EXCHANGE,
           "Cryptopia Withdrawals",
           ['#', 'Currency', 'Amount', 'Fee', 'Status', 'TransactionId', 'Address', 'Timestamp'],
           worksheet_name="Cryptopia W",
           row_handler=parse_cryptopia_withdrawals)

DataParser(DataParser.TYPE_EXCHANGE,
           "Cryptopia Trades",
           ['#', 'Market', 'Type', 'Rate', 'Amount', 'Total', 'Fee', 'Timestamp'],
           worksheet_name="Cryptopia T",
           row_handler=parse_cryptopia_trades)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnknownCryptoassetError, UnexpectedTypeError

WALLET = "Trezor"

def parse_trezor(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0] + 'T' + in_row[1])

    if not config.args.cryptoasset:
        raise UnknownCryptoassetError

    if in_row[5] == "IN":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[7],
                                                 buy_asset=config.args.cryptoasset,
                                                 wallet=WALLET)
    elif in_row[5] == "OUT" or in_row[5] == "SELF":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[7])),
                                                 sell_asset=config.args.cryptoasset,
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(5, parser.in_header[5], in_row[5])

DataParser(DataParser.TYPE_WALLET,
           "Trezor",
           ['Date', 'Time', 'TX id', 'Address', 'Address Label', 'TX type', 'Value', 'TX total',
            'Balance'],
           worksheet_name="Trezor",
           row_handler=parse_trezor)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2020

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "KuCoin"

def parse_kucoin_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0], tz='Asia/Hong_Kong')

    if in_row[3] == "buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[2].split('-')[0],
                                                 sell_quantity=in_row[6],
                                                 sell_asset=in_row[2].split('-')[1],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[9],
                                                 wallet=WALLET)
    elif in_row[3] == "sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[6],
                                                 buy_asset=in_row[2].split('-')[1],
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[2].split('-')[0],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[9],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(3, parser.in_header[3], in_row[3])

DataParser(DataParser.TYPE_EXCHANGE,
           "KuCoin",
           ['tradeCreatedAt', 'orderId', 'symbol', 'side', 'price', 'size', 'funds', 'fee',
            'liquidity', 'feeCurrency', 'orderType'],
           worksheet_name="KuCoin T",
           row_handler=parse_kucoin_trades)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Coinbase Pro"

def parse_coinbase_pro_deposits_withdrawals(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[1])

    if in_row[0] == "withdrawal":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[2])),
                                                 sell_asset=in_row[4],
                                                 wallet=WALLET)
    elif in_row[0] == "deposit":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[2],
                                                 buy_asset=in_row[4],
                                                 wallet=WALLET)
    elif in_row[0] in ("match", "fee"):
        # Skip trades
        return
    else:
        raise UnexpectedTypeError(0, parser.in_header[0], in_row[0])

def parse_coinbase_pro_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[3])

    if in_row[2] == "BUY":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[4],
                                                 buy_asset=in_row[5],
                                                 sell_quantity=abs(Decimal(in_row[8])) - \
                                                               Decimal(in_row[7]),
                                                 sell_asset=in_row[9],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[9],
                                                 wallet=WALLET)
    elif in_row[2] == "SELL":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[8],
                                                 buy_asset=in_row[9],
                                                 sell_quantity=in_row[4],
                                                 sell_asset=in_row[5],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[9],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinbase Pro Deposits/Withdrawals",
           ['type', 'time', 'amount', 'balance', 'amount/balance unit', 'transfer id', 'trade id',
            'order id'],
           worksheet_name="Coinbase Pro D,W",
           row_handler=parse_coinbase_pro_deposits_withdrawals)

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinbase Pro Trades",
           ['trade id', 'product', 'side', 'created at', 'size', 'size unit', 'price', 'fee',
            'total', 'price/fee/total unit'],
           worksheet_name="Coinbase Pro T",
           row_handler=parse_coinbase_pro_trades)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Bitstamp"

def parse_bitstamp(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[1])

    if in_row[0] in ("Ripple deposit", "Deposit"):
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3].split(' ')[0],
                                                 buy_asset=in_row[3].split(' ')[1],
                                                 wallet=WALLET)
    elif in_row[0] in ("Ripple payment", "Withdrawal"):
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[3].split(' ')[0],
                                                 sell_asset=in_row[3].split(' ')[1],
                                                 wallet=WALLET)
    elif in_row[0] == "Market":
        if in_row[7] == "Buy":
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                     data_row.timestamp,
                                                     buy_quantity=in_row[3].split(' ')[0],
                                                     buy_asset=in_row[3].split(' ')[1],
                                                     sell_quantity=in_row[4].split(' ')[0],
                                                     sell_asset=in_row[4].split(' ')[1],
                                                     fee_quantity=in_row[6].split(' ')[0],
                                                     fee_asset=in_row[6].split(' ')[1],
                                                     wallet=WALLET)
        elif in_row[7] == "Sell":
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                     data_row.timestamp,
                                                     buy_quantity=in_row[4].split(' ')[0],
                                                     buy_asset=in_row[4].split(' ')[1],
                                                     sell_quantity=in_row[3].split(' ')[0],
                                                     sell_asset=in_row[3].split(' ')[1],
                                                     fee_quantity=in_row[6].split(' ')[0],
                                                     fee_asset=in_row[6].split(' ')[1],
                                                     wallet=WALLET)
        else:
            raise UnexpectedTypeError(7, parser.in_header[7], in_row[7])
    else:
        raise UnexpectedTypeError(0, parser.in_header[0], in_row[0])

DataParser(DataParser.TYPE_EXCHANGE,
           "Bitstamp",
           ['Type', 'Datetime', 'Account', 'Amount', 'Value', 'Rate', 'Fee', 'Sub Type'],
           worksheet_name="Bitstamp",
           row_handler=parse_bitstamp)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnknownUsernameError

WALLET = "ChangeTip"

def parse_changetip(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[3])

    if in_row[6] == "Delivered":
        if in_row[2] in config.usernames:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_GIFT_RECEIVED,
                                                     data_row.timestamp,
                                                     buy_quantity=Decimal(in_row[4]) / 100000000,
                                                     buy_asset="BTC",
                                                     wallet=WALLET)
        elif in_row[1] in config.usernames:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_GIFT_SENT,
                                                     data_row.timestamp,
                                                     sell_quantity=Decimal(in_row[4]) / 100000000,
                                                     sell_asset="BTC",
                                                     wallet=WALLET)
        else:
            raise UnknownUsernameError
    else:
        return

DataParser(DataParser.TYPE_EXCHANGE,
           "ChangeTip",
           ['On', 'From', 'To', 'When', 'Amount in Satoshi', 'mBTC', 'Status', 'Message'],
           worksheet_name="ChangeTip",
           row_handler=parse_changetip)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Uphold"

def parse_uphold(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[2] == "deposit" or in_row[2] == "in":
        # Check if origin and destination currency are the same
        if in_row[7] == in_row[10]:
            fee_quantity = Decimal(in_row[8]) - Decimal(in_row[11])
            fee_asset = in_row[7]
        else:
            fee_quantity = None
            fee_asset = ""

        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[8],
                                                 buy_asset=in_row[7],
                                                 fee_quantity=fee_quantity,
                                                 fee_asset=fee_asset,
                                                 wallet=WALLET)
    elif in_row[2] == "withdrawal" or in_row[2] == "out":
        # Check if origin and destination currency are the same
        if in_row[7] == in_row[10]:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                     data_row.timestamp,
                                                     sell_quantity=in_row[11],
                                                     sell_asset=in_row[7],
                                                     fee_quantity=Decimal(in_row[8]) - \
                                                                  Decimal(in_row[11]),
                                                     fee_asset=in_row[7],
                                                     wallet=WALLET)
        else:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                     data_row.timestamp,
                                                     sell_quantity=in_row[8],
                                                     sell_asset=in_row[7],
                                                     wallet=WALLET)
    elif in_row[2] == "transfer":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[11],
                                                 buy_asset=in_row[10],
                                                 sell_quantity=in_row[8],
                                                 sell_asset=in_row[7],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

DataParser(DataParser.TYPE_EXCHANGE,
           "Uphold",
           ['date', 'id', 'type', 'value_in_GBP', 'commission_in_GBP', 'pair', 'rate',
            'origin_currency', 'origin_amount', 'origin_commission', 'destination_currency',
            'destination_amount', 'destination_commission'],
           worksheet_name="Uphold",
           row_handler=parse_uphold)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Coinbase"
DUPLICATE = "Duplicate"

def parse_coinbase_transfers(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[1] == "Deposit":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[6],
                                                 fee_quantity=in_row[4],
                                                 fee_asset=in_row[6],
                                                 wallet=WALLET)
    elif in_row[1] == "Withdrawal":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[6],
                                                 fee_quantity=in_row[4],
                                                 fee_asset=in_row[6],
                                                 wallet=WALLET)
    elif in_row[1] == "Buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[2],
                                                 buy_asset="BTC",
                                                 sell_quantity=in_row[3],
                                                 sell_asset=in_row[6],
                                                 fee_quantity=in_row[4],
                                                 fee_asset=in_row[6],
                                                 wallet=WALLET)
    elif in_row[1] == "Sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3],
                                                 buy_asset=in_row[6],
                                                 sell_quantity=in_row[2],
                                                 sell_asset="BTC",
                                                 fee_quantity=in_row[4],
                                                 fee_asset=in_row[6],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(1, parser.in_header[1], in_row[1])

def parse_coinbase_transactions(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[21] != "":
        # Hash so must be external crypto deposit or withdrawal
        if Decimal(in_row[2]) < 0:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                     data_row.timestamp,
                                                     sell_quantity=abs(Decimal(in_row[2])),
                                                     sell_asset=in_row[3],
                                                     wallet=WALLET)
        else:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                     data_row.timestamp,
                                                     buy_quantity=in_row[2],
                                                     buy_asset=in_row[3],
                                                     wallet=WALLET)
    elif in_row[12] != "":
        # Transfer ID so could be a trade or external fiat deposit/withdrawal
        if in_row[3] != in_row[8]:
            # Currencies are different so must be a trade
            if Decimal(in_row[2]) < 0:
                data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                         data_row.timestamp,
                                                         buy_quantity=Decimal(in_row[7]) + \
                                                                      Decimal(in_row[9]),
                                                         buy_asset=in_row[8],
                                                         sell_quantity=abs(Decimal(in_row[2])),
                                                         sell_asset=in_row[3],
                                                         fee_quantity=in_row[9],
                                                         fee_asset=in_row[10],
                                                         wallet=WALLET)
            else:
                data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                         data_row.timestamp,
                                                         buy_quantity=in_row[2],
                                                         buy_asset=in_row[3],
                                                         sell_quantity=Decimal(in_row[7]) - \
                                                                       Decimal(in_row[9]),
                                                         sell_asset=in_row[8],
                                                         fee_quantity=in_row[9],
                                                         fee_asset=in_row[10],
                                                         wallet=WALLET)
        else:
            if Decimal(in_row[2]) < 0:
                data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                         data_row.timestamp,
                                                         sell_quantity=in_row[7],
                                                         sell_asset=in_row[3],
                                                         fee_quantity=in_row[9],
                                                         fee_asset=in_row[10],
                                                         wallet=WALLET)
            else:
                data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                         data_row.timestamp,
                                                         buy_quantity=in_row[7],
                                                         buy_asset=in_row[3],
                                                         fee_quantity=in_row[9],
                                                         fee_asset=in_row[10],
                                                         wallet=WALLET)
    else:
        # Could be a referral bonus or deposit/withdrawal to/from Coinbase Pro
        if in_row[5] != "" and in_row[3] == "BTC":
            # Bonus is always in BTC
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_GIFT_RECEIVED,
                                                     data_row.timestamp,
                                                     buy_quantity=in_row[2],
                                                     buy_asset=in_row[3],
                                                     wallet=WALLET)
        elif in_row[5] != "" and in_row[3] != "BTC":
            # Special case, flag as duplicate entry, trade will be in BTC Wallet Transactions Report
            if Decimal(in_row[2]) < 0:
                data_row.t_record = TransactionOutRecord(DUPLICATE,
                                                         data_row.timestamp,
                                                         sell_quantity=abs(Decimal(in_row[2])),
                                                         sell_asset=in_row[3],
                                                         wallet=WALLET)
            else:
                data_row.t_record = TransactionOutRecord(DUPLICATE,
                                                         data_row.timestamp,
                                                         buy_quantity=in_row[2],
                                                         buy_asset=in_row[3],
                                                         wallet=WALLET)
        elif Decimal(in_row[2]) < 0:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                     data_row.timestamp,
                                                     sell_quantity=abs(Decimal(in_row[2])),
                                                     sell_asset=in_row[3],
                                                     wallet=WALLET)
        else:
            data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                     data_row.timestamp,
                                                     buy_quantity=in_row[2],
                                                     buy_asset=in_row[3],
                                                     wallet=WALLET)

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinbase Transfers",
           ['Timestamp', 'Type', None, 'Subtotal', 'Fees', 'Total', 'Currency', 'Price Per Coin',
            'Payment Method', 'ID', 'Share'],
           worksheet_name="Coinbase",
           row_handler=parse_coinbase_transfers)

DataParser(DataParser.TYPE_EXCHANGE,
           "Coinbase Transactions",
           ['Timestamp', 'Balance', 'Amount', 'Currency', 'To', 'Notes', 'Instantly Exchanged',
            'Transfer Total', 'Transfer Total Currency', 'Transfer Fee', 'Transfer Fee Currency',
            'Transfer Payment Method', 'Transfer ID', 'Order Price', 'Order Currency', None,
            'Order Tracking Code', 'Order Custom Parameter', 'Order Paid Out',
            'Recurring Payment ID', None, None],
           worksheet_name="Coinbase",
           row_handler=parse_coinbase_transactions)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser

WALLET = "Ethereum"

def parse_etherscan(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(int(in_row[2]))

    if Decimal(in_row[7]) > 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[7],
                                                 buy_asset="ETH",
                                                 wallet=WALLET)
    elif Decimal(in_row[8]) > 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[8],
                                                 sell_asset="ETH",
                                                 fee_quantity=in_row[10],
                                                 fee_asset="ETH",
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_SPEND,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[8],
                                                 sell_asset="ETH",
                                                 fee_quantity=in_row[10],
                                                 fee_asset="ETH",
                                                 wallet=WALLET)

DataParser(DataParser.TYPE_EXPLORER,
           "Etherscan (Ethereum)",
           ['Txhash', 'Blockno', 'UnixTimestamp', 'DateTime', 'From', 'To', 'ContractAddress',
            'Value_IN(ETH)', 'Value_OUT(ETH)', None, 'TxnFee(ETH)', 'TxnFee(USD)',
            'Historical $Price/Eth', 'Status', 'ErrCode'],
           worksheet_name="Etherscan",
           row_handler=parse_etherscan)

DataParser(DataParser.TYPE_EXPLORER,
           "Etherscan (Ethereum)",
           ['Txhash', 'Blockno', 'UnixTimestamp', 'DateTime', 'From', 'To', 'ContractAddress',
            'Value_IN(ETH)', 'Value_OUT(ETH)', None, 'TxnFee(ETH)', 'TxnFee(USD)',
            'Historical $Price/Eth', 'Status', 'ErrCode', 'PrivateNote'],
           worksheet_name="Etherscan",
           row_handler=parse_etherscan)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser

WALLET = "Bitfinex"

def parse_bitfinex_trades(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[6], dayfirst=True)

    if Decimal(in_row[2]) > 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[2],
                                                 buy_asset=in_row[1].split('/')[0],
                                                 sell_quantity=Decimal(in_row[3]) * \
                                                                       Decimal(in_row[2]),
                                                 sell_asset=in_row[1].split('/')[1],
                                                 fee_quantity=abs(Decimal(in_row[4])),
                                                 fee_asset=in_row[5],
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[3]) * \
                                                                      abs(Decimal(in_row[2])),
                                                 buy_asset=in_row[1].split('/')[1],
                                                 sell_quantity=abs(Decimal(in_row[2])),
                                                 sell_asset=in_row[1].split('/')[0],
                                                 fee_quantity=abs(Decimal(in_row[4])),
                                                 fee_asset=in_row[5],
                                                 wallet=WALLET)

def parse_bitfinex_deposits_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[1], dayfirst=True)

    if in_row[3] != "COMPLETED":
        return

    if Decimal(in_row[4]) > 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[4],
                                                 buy_asset=in_row[2],
                                                 fee_quantity=abs(Decimal(in_row[5])),
                                                 fee_asset=in_row[2],
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[4])),
                                                 sell_asset=in_row[2],
                                                 fee_quantity=abs(Decimal(in_row[5])),
                                                 fee_asset=in_row[2],
                                                 wallet=WALLET)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bitfinex Trades",
           ['#', 'PAIR', 'AMOUNT', 'PRICE', 'FEE', 'FEE CURRENCY', 'DATE', 'ORDER ID'],
           worksheet_name="Bitfinex T",
           row_handler=parse_bitfinex_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bitfinex Deposits/Withdrawals",
           ['#', 'DATE', 'CURRENCY', 'STATUS', 'AMOUNT', 'FEES', 'DESCRIPTION', 'TRANSACTION ID'],
           worksheet_name="Bitfinex D,W",
           row_handler=parse_bitfinex_deposits_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Wirex"

def parse_wirex(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[2])

    if in_row[1] == "Create":
        return
    elif in_row[1] == "In":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3].split(' ')[0],
                                                 buy_asset=in_row[3].split(' ')[1],
                                                 wallet=WALLET)
    elif in_row[1] == "Out":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[3].split(' ')[0],
                                                 sell_asset=in_row[3].split(' ')[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(1, parser.in_header[1], in_row[1])

DataParser(DataParser.TYPE_EXCHANGE,
           "Wirex",
           ['# ', None, 'Time ', 'Amount', 'Available'],
           worksheet_name="Wirex",
           row_handler=parse_wirex)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnknownCryptoassetError

WALLET = "Electrum"

def parse_electrum(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[4])

    if not config.args.cryptoasset:
        raise UnknownCryptoassetError

    if Decimal(in_row[3]) > 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[3]),
                                                 buy_asset=config.args.cryptoasset,
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[3])),
                                                 sell_asset=config.args.cryptoasset,
                                                 wallet=WALLET)

DataParser(DataParser.TYPE_WALLET,
           "Electrum",
           ['transaction_hash', 'label', 'confirmations', 'value', 'timestamp'],
           worksheet_name="Electrum",
           row_handler=parse_electrum)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError, UnexpectedTradingPairError

WALLET = "Binance"
QUOTE_ASSETS = ['BNB', 'BTC', 'BUSD', 'ETH', 'EUR', 'GBP', 'NGN', 'PAX', 'RUB', 'TRX',
                'TUSD', 'USDC', 'USDS', 'USDT', 'XRP']

def parse_binance_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    base_asset, quote_asset = split_trading_pair(in_row[1])
    if base_asset is None or quote_asset is None:
        raise UnexpectedTradingPairError(1, parser.in_header[1], in_row[1])

    if in_row[2] == "BUY":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[4],
                                                 buy_asset=base_asset,
                                                 sell_quantity=in_row[5],
                                                 sell_asset=quote_asset,
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[7],
                                                 wallet=WALLET)
    elif in_row[2] == "SELL":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=quote_asset,
                                                 sell_quantity=in_row[4],
                                                 sell_asset=base_asset,
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[7],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

def parse_binance_deposits_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[8] != "Completed":
        return

    # Assume that a transaction fee of 0 must be a Deposit
    if Decimal(in_row[3]) == 0:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[2],
                                                 buy_asset=in_row[1],
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=in_row[2],
                                                 sell_asset=in_row[1],
                                                 fee_quantity=in_row[3],
                                                 fee_asset=in_row[1],
                                                 wallet=WALLET)
def split_trading_pair(trading_pair):
    for quote_asset in QUOTE_ASSETS:
        if trading_pair.endswith(quote_asset):
            return trading_pair.replace(quote_asset, ''), quote_asset

    return None, None

DataParser(DataParser.TYPE_EXCHANGE,
           "Binance Trades",
           ['Date(UTC)', 'Market', 'Type', 'Price', 'Amount', 'Total', 'Fee', 'Fee Coin'],
           worksheet_name="Binance T",
           row_handler=parse_binance_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "Binance Deposits/Withdrawals",
           ['Date', 'Coin', 'Amount', 'TransactionFee', 'Address', 'TXID', 'SourceAddress',
            'PaymentID', 'Status'],
           worksheet_name="Binance D,W",
           row_handler=parse_binance_deposits_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

import re
from decimal import Decimal

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnknownCryptoassetError, UnexpectedTypeError

WALLET = "Qt Wallet"

def parse_qt_wallet(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[1], tz='Europe/London')

    if parser.args[0].group(2):
        symbol = parser.args[0].group(2)
    elif config.args.cryptoasset:
        symbol = config.args.cryptoasset
    else:
        raise UnknownCryptoassetError

    if in_row[0] == "false" and not config.args.unconfirmed:
        # skip unconfirmed transactions
        return

    if in_row[2] == "Received with":
        t_type = TransactionOutRecord.TYPE_DEPOSIT
    elif in_row[2] == "Sent to":
        t_type = TransactionOutRecord.TYPE_WITHDRAWAL
    elif in_row[2] == "Mined":
        t_type = TransactionOutRecord.TYPE_MINING
    elif in_row[2] == "Payment to yourself":
        t_type = TransactionOutRecord.TYPE_WITHDRAWAL
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

    if Decimal(in_row[5]) > 0:
        data_row.t_record = TransactionOutRecord(t_type,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=symbol,
                                                 wallet=WALLET)
    else:
        data_row.t_record = TransactionOutRecord(t_type,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[5])),
                                                 sell_asset=symbol,
                                                 wallet=WALLET)

DataParser(DataParser.TYPE_WALLET,
           "Qt Wallet (i.e. Bitcoin Core, etc)",
           ['Confirmed', 'Date', 'Type', 'Label', 'Address',
            lambda c: re.match(r"Amount( \((\w+)\))?", c), 'ID'],
           worksheet_name="Qt Wallet",
           row_handler=parse_qt_wallet)



from . import binance
from . import barclays
from . import bitfinex
from . import bitstamp
from . import bittrex
from . import changetip
from . import circle
from . import coinbase
from . import coinbasepro
from . import coinfloor
from . import cryptopia
from . import cryptsy
from . import electrum
from . import etherscan
from . import gatehub
from . import hitbtc
from . import kucoin
from . import ledgerlive
from . import okex
from . import poloniex
from . import qtwallet
from . import tradesatoshi
from . import trezor
from . import uphold
from . import wirex



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Ledger Live"

def parse_ledger_live(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[2] == "IN":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[3]) + \
                                                              Decimal(in_row[4]),
                                                 buy_asset=in_row[1],
                                                 fee_quantity=Decimal(in_row[4]),
                                                 fee_asset=in_row[1],
                                                 wallet=WALLET)

    if in_row[2] == "OUT":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=Decimal(in_row[3]) - \
                                                               Decimal(in_row[4]),
                                                 sell_asset=in_row[1],
                                                 fee_quantity=Decimal(in_row[4]),
                                                 fee_asset=in_row[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

DataParser(DataParser.TYPE_WALLET,
           "Ledger Live",
           ['Operation Date', 'Currency Ticker', 'Operation Type', 'Operation Amount',
            'Operation Fees', 'Operation Hash', 'Account Name', 'Account id'],
           worksheet_name="Ledger",
           row_handler=parse_ledger_live)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2020

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "HitBTC"

def parse_hitbtc_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[4] == "buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[1].split('/')[0],
                                                 sell_quantity=Decimal(in_row[7]) - \
                                                               Decimal(in_row[9]),
                                                 sell_asset=in_row[1].split('/')[1],
                                                 fee_quantity=in_row[8],
                                                 fee_asset=in_row[1].split('/')[1],
                                                 wallet=WALLET)
    elif in_row[4] == "sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[7]) + \
                                                              Decimal(in_row[9]),
                                                 buy_asset=in_row[1].split('/')[1],
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[1].split('/')[0],
                                                 fee_quantity=in_row[8],
                                                 fee_asset=in_row[1].split('/')[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(4, parser.in_header[4], in_row[4])

def parse_hitbtc_deposits_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[0])

    if in_row[2] == "Withdraw":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                                 data_row.timestamp,
                                                 sell_quantity=abs(Decimal(in_row[3])),
                                                 sell_asset=in_row[6],
                                                 wallet=WALLET)
    elif in_row[2] == "Deposit":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3],
                                                 buy_asset=in_row[6],
                                                 wallet=WALLET)

DataParser(DataParser.TYPE_EXCHANGE,
           "HitBTC",
           ['Date (UTC)', 'Instrument', 'Trade ID', 'Order ID', 'Side', 'Quantity', 'Price',
            'Volume', 'Fee', 'Rebate', 'Total'],
           worksheet_name="HitBTC T",
           row_handler=parse_hitbtc_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "HitBTC",
           ['Date (UTC)', 'Operation id', 'Type', 'Amount', 'Transaction Hash',
            'Main account balance'],
           worksheet_name="HitBTC D,W",
           row_handler=parse_hitbtc_deposits_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

import dateutil.tz

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import DataParserError, UnexpectedTypeError

WALLET = "OKEx"
TZ_INFOS = {'CST': dateutil.tz.gettz('Asia/Shanghai')}

def parse_okex_trades(data_rows, parser):
    for buy_row, sell_row in zip(data_rows[0::2], data_rows[1::2]):
        try:
            parse_okex_trades_row(buy_row, sell_row, parser)
        except DataParserError as e:
            buy_row.failure = e

def parse_okex_trades_row(buy_row, sell_row, parser):
    buy_row.timestamp = DataParser.parse_timestamp(buy_row.in_row[0], tzinfos=TZ_INFOS)
    sell_row.timestamp = DataParser.parse_timestamp(sell_row.in_row[0], tzinfos=TZ_INFOS)

    if buy_row.in_row[1] == "buy" and sell_row.in_row[1] == "sell":
        buy_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                buy_row.timestamp,
                                                buy_quantity=buy_row.in_row[2],
                                                buy_asset=buy_row.in_row[5],
                                                sell_quantity=abs(Decimal(sell_row.in_row[2])),
                                                sell_asset=sell_row.in_row[5],
                                                fee_quantity=abs(Decimal(buy_row.in_row[4])),
                                                fee_asset=buy_row.in_row[5],
                                                wallet=WALLET)
    else:
        raise UnexpectedTypeError(1, parser.in_header[1], buy_row.in_row[1])

DataParser(DataParser.TYPE_EXCHANGE,
           "OKEx",
           ['time', 'type', 'size', 'balance', 'fee', 'currency'],
           worksheet_name="OKEx",
           all_handler=parse_okex_trades)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ...config import config
from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

def parse_barclays(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[1])

    if in_row[2] != "Completed":
        return

    if in_row[4] == "Buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[5],
                                                 buy_asset=in_row[0],
                                                 sell_quantity=in_row[6],
                                                 sell_asset=config.CCY,
                                                 wallet=in_row[3])
    elif in_row[4] == "Sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[6],
                                                 buy_asset=config.CCY,
                                                 sell_quantity=in_row[5],
                                                 sell_asset=in_row[0],
                                                 wallet=in_row[3])
    else:
        raise UnexpectedTypeError(4, parser.in_header[4], in_row[4])

DataParser(DataParser.TYPE_SHARES,
           "Barclays Smart Investor",
           ['Investment', 'Date', 'Order Status', 'Account', 'Buy/Sell', 'Quantity',
            'Cost/Proceeds'],
           worksheet_name="Barclays",
           row_handler=parse_barclays)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from decimal import Decimal

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Bittrex"

def parse_bittrex_trades2(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[14])

    if in_row[3] == "LIMIT_BUY":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=Decimal(in_row[5]) - \
                                                              Decimal(in_row[6]),
                                                 buy_asset=in_row[1].split('-')[1],
                                                 sell_quantity=in_row[8],
                                                 sell_asset=in_row[1].split('-')[0],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[1].split('-')[0],
                                                 wallet=WALLET)
    elif in_row[3] == "LIMIT_SELL":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[8],
                                                 buy_asset=in_row[1].split('-')[0],
                                                 sell_quantity=Decimal(in_row[5]) - \
                                                               Decimal(in_row[6]),
                                                 sell_asset=in_row[1].split('-')[1],
                                                 fee_quantity=in_row[7],
                                                 fee_asset=in_row[1].split('-')[0],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(3, parser.in_header[3], in_row[3])

def parse_bittrex_trades(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[8])

    if in_row[2] == "LIMIT_BUY":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[3],
                                                 buy_asset=in_row[1].split('-')[1],
                                                 sell_quantity=in_row[6],
                                                 sell_asset=in_row[1].split('-')[0],
                                                 fee_quantity=in_row[5],
                                                 fee_asset=in_row[1].split('-')[0],
                                                 wallet=WALLET)
    elif in_row[2] == "LIMIT_SELL":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[6],
                                                 buy_asset=in_row[1].split('-')[0],
                                                 sell_quantity=in_row[3],
                                                 sell_asset=in_row[1].split('-')[1],
                                                 fee_quantity=in_row[5],
                                                 fee_asset=in_row[1].split('-')[0],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(2, parser.in_header[2], in_row[2])

def parse_bittrex_deposits2(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[4])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                             data_row.timestamp,
                                             buy_quantity=in_row[2],
                                             buy_asset=in_row[1],
                                             wallet=WALLET)

def parse_bittrex_deposits(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[4])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_DEPOSIT,
                                             data_row.timestamp,
                                             buy_quantity=in_row[1],
                                             buy_asset=in_row[2],
                                             wallet=WALLET)

def parse_bittrex_withdrawals(data_row, _):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[4])

    data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_WITHDRAWAL,
                                             data_row.timestamp,
                                             sell_quantity=in_row[2],
                                             sell_asset=in_row[1],
                                             fee_quantity=in_row[7],
                                             fee_asset=in_row[1],
                                             wallet=WALLET)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Trades",
           ['Uuid', 'Exchange', 'TimeStamp', 'OrderType', 'Limit', 'Quantity', 'QuantityRemaining',
            'Commission', 'Price', 'PricePerUnit', 'IsConditional', 'Condition', 'ConditionTarget',
            'ImmediateOrCancel', 'Closed'],
           worksheet_name="Bittrex T",
           row_handler=parse_bittrex_trades2)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Trades",
           ['OrderUuid', 'Exchange', 'Type', 'Quantity', 'Limit', 'CommissionPaid', 'Price',
            'Opened', 'Closed'],
           worksheet_name="Bittrex T",
           row_handler=parse_bittrex_trades)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Deposits",
           ['Id', 'Currency', 'Amount', 'Confirmations', 'LastUpdatedDate', 'TxId',
            'CryptoAddress'],
           worksheet_name="Bittrex D",
           row_handler=parse_bittrex_deposits2)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Deposits",
           ['Id', 'Amount', 'Currency', 'Confirmations', 'LastUpdated', 'TxId', 'CryptoAddress'],
           worksheet_name="Bittrex D",
           row_handler=parse_bittrex_deposits)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Withdrawals",
           ['PaymentUuid', 'Currency', 'Amount', 'Address', 'OpenedDate', 'Authorized', 'Pending',
            'TxFee', 'Canceled', 'TxId'],
           worksheet_name="Bittrex W",
           row_handler=parse_bittrex_withdrawals)

DataParser(DataParser.TYPE_EXCHANGE,
           "Bittrex Withdrawals",
           ['PaymentUuid', 'Currency', 'Amount', 'Address', 'Opened', 'Authorized',
            'PendingPayment', 'TxCost', 'TxId', 'Canceled', 'InvalidAddress'],
           worksheet_name="Bittrex W",
           row_handler=parse_bittrex_withdrawals)



# -*- coding: utf-8 -*-
# (c) Nano Nano Ltd 2019

from ..out_record import TransactionOutRecord
from ..dataparser import DataParser
from ..exceptions import UnexpectedTypeError

WALLET = "Crypsty"

def parse_cryptsy(data_row, parser):
    in_row = data_row.in_row
    data_row.timestamp = DataParser.parse_timestamp(in_row[8], tz='US/Eastern')

    if in_row[1] == "Buy":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[4],
                                                 buy_asset=in_row[2].split('/')[0],
                                                 sell_quantity=in_row[7],
                                                 sell_asset=in_row[2].split('/')[1],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[2].split('/')[1],
                                                 wallet=WALLET)
    elif in_row[1] == "Sell":
        data_row.t_record = TransactionOutRecord(TransactionOutRecord.TYPE_TRADE,
                                                 data_row.timestamp,
                                                 buy_quantity=in_row[7],
                                                 buy_asset=in_row[2].split('/')[1],
                                                 sell_quantity=in_row[4],
                                                 sell_asset=in_row[2].split('/')[0],
                                                 fee_quantity=in_row[6],
                                                 fee_asset=in_row[2].split('/')[1],
                                                 wallet=WALLET)
    else:
        raise UnexpectedTypeError(1, parser.in_header[1], in_row[1])

DataParser(DataParser.TYPE_EXCHANGE,
           "Cryptsy",
           ['TradeID', 'OrderType', 'Market', 'Price', 'Quantity', 'Total', 'Fee', 'Net',
            'Timestamp'],
           worksheet_name="Cryptsy",
           row_handler=parse_cryptsy)
